<?php

namespace seraph_accel;

if( !defined( 'ABSPATH' ) )
	exit;

class Db
{
	static function GetTblPrefix( $name = '' )
	{
		global $wpdb;
		return( $wpdb -> prefix . $name );
	}
}

class DbTbl
{
	static function CreateUpdate( $id, $cols )
	{
		global $wpdb;

		$charset_collate = $wpdb -> get_charset_collate();

		$queryCols = '';
		$querySuffix = '';

		foreach( $cols as $colId => $col )
		{
			$queryColsAttr = '' . $colId . ' ' . strtolower( $col[ 'type' ] );

			$colAttrs = @$col[ 'attrs' ];
			if( $colAttrs )
			{
				foreach( $colAttrs as $colAttr )
				{
					$querySuffixAttr = null;
					switch( $colAttr )
					{
					case 'PRIMARY KEY':
						$querySuffixAttr = 'PRIMARY KEY (' . $colId . ')';
						break;

					case 'KEY':
					case 'INDEX':
						$querySuffixAttr = 'KEY (' . $colId . ')';
						break;

					default:
						$queryColsAttr .= ' ' . $colAttr;
						break;
					}

					if( $querySuffixAttr )
						self::_AddCommaArg( $querySuffix, $querySuffixAttr );
				}
			}

			self::_AddCommaArg( $queryCols, $queryColsAttr, "\n" );
		}

		$query = 'CREATE TABLE ' . $id . ' (' . "\n" . $queryCols;
		self::_AddCommaArg( $query, $querySuffix, "\n" );
		$query .= "\n" . ') ' . $charset_collate . ';';

		require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
		dbDelta( $query );
	}

	static function Delete( $id )
	{
		global $wpdb;
		return( $wpdb -> query( 'DROP TABLE ' . $id . '' ) );
	}

	static function GetRows( $id, $cols = null, $limit = null, $where = null, $order = null, $output = ARRAY_A )
	{
		global $wpdb;

		if( $where )
		{
			$where = self::_process_fields( $id, $where, self::_GetFormat( $where ) );
			if( false === $where )
				return( false );
		}

		$fields = array();
		$conditions = array();
		$values = array();

		if( is_array( $cols ) )
		{
			foreach( $cols as $field )
				$fields[] = "`$field`";
		}
		else
			$fields[] = '*';

		if( $where )
		{
			foreach( $where as $field => $value )
			{
				if( is_null( $value[ 'value' ] ) )
				{
					$conditions[] = "`$field` IS NULL";
					continue;
				}

				$conditions[] = "`$field` = " . $value[ 'format' ];
				$values[] = $value[ 'value' ];
			}
		}

		$fields = implode( ', ', $fields );
		$conditions = implode( ' AND ', $conditions );

		$sql = "SELECT $fields FROM `$id`";
		if( $conditions )
			$sql .= " WHERE $conditions";

		if( $order )
		{
			$orderArgs = array();
			foreach( $order as $orderCol => $orderColDir )
				$orderArgs[] = "`$orderCol` $orderColDir";
			if( $orderArgs )
			{
				$orderArgs = implode( ', ', $orderArgs );
				$sql .= " ORDER BY $orderArgs";
			}
		}

		if( $limit )
			$sql .= ' LIMIT ' . $limit;

		return( $wpdb -> get_results( $values ? $wpdb -> prepare( $sql, $values ) : $sql, $output ) );
	}

	static function InsertRow( $id, $data )
	{
		global $wpdb;
		$suppress = $wpdb -> suppress_errors();
		$res = $wpdb -> insert( $id, $data );
		$wpdb -> suppress_errors( $suppress );
		return( $res );
	}

	static function UpdateRows( $id, $data, $where )
	{
		global $wpdb;
		return( $wpdb -> update( $id, $data, $where, self::_GetFormat( $data ), self::_GetFormat( $where ) ) );
	}

	static function DeleteRows( $id, $where = null )
	{
		global $wpdb;

		if( !$where )
			return( $wpdb -> query( 'DELETE FROM `' . $id . '`' ) );
		return( $wpdb -> delete( $id, $where, self::_GetFormat( $where ) ) );
	}

	private static function _AddCommaArg( &$str, $arg, $commaSuffix = '' )
	{
		if( empty( $arg ) )
			return;

		if( !empty( $str ) )
			$str .= ',' . $commaSuffix;
		$str .= $arg;
	}

	protected static function _GetFormat( $data )
	{
		$formats = array();

		foreach( $data as $field => $value )
		{
			$fmt = '%s';
			switch( gettype( $value ) )
			{
			case 'integer':	$fmt = '%d'; break;
			case 'boolean':	$fmt = '%u'; break;
			case 'double':	$fmt = '%f'; break;
			}

			$formats[] = $fmt;
		}

		return( $formats );
	}

	protected static function _process_fields( $table, $data, $format )
	{
		global $wpdb;

		$formats          = (array) $format;
		$original_formats = $formats;

		foreach( $data as $field => $value )
		{
			$value = array(
				'value'  => $value,
				'format' => '%s',
			);

			if ( ! empty( $format ) ) {
				$value['format'] = array_shift( $formats );
				if ( ! $value['format'] ) {
					$value['format'] = reset( $original_formats );
				}
			} elseif ( isset( $wpdb->field_types[ $field ] ) ) {
				$value['format'] = $wpdb->field_types[ $field ];
			}

			$data[ $field ] = $value;
		}

		return $data;
	}
}

class DbTran
{

	const IsolLvl_Def			= null;
	const IsolLvl_RepRead		= 'REPEATABLE READ';
	const IsolLvl_ReadComm		= 'READ COMMITTED';
	const IsolLvl_ReadUncomm	= 'READ UNCOMMITTED';
	const IsolLvl_Ser			= 'SERIALIZABLE';

	function __destruct()
	{
		$this -> Close();
	}

	function Open( $isolLvl = self::IsolLvl_Def )
	{
		global $wpdb;

		if( $this -> _active )
			return( true );

		$res = true;
		if( $isolLvl )
		    $res = $wpdb -> query( 'SET TRANSACTION ISOLATION LEVEL ' . $isolLvl );

		if( $res !== false )
			$res = $wpdb -> query( 'START TRANSACTION' );

		if( $res !== false )
			$this -> _active = true;
		return( $res );
	}

	function Close( $commit = false )
	{
		global $wpdb;

		if( !$this -> _active )
			return( true );

		$res = $wpdb -> query( $commit ? 'COMMIT' : 'ROLLBACK' );
		if( $res !== false )
			$this -> _active = false;
		return( $res );
	}

	private $_active = false;
}

